import functools

from flask import request, current_app
from werkzeug.exceptions import NotAcceptable


class ExtendHeaders(object):
    def __init__(self, app=None, header: str = ""):
        self.app = app
        self.header = header
        if app is not None:
            self.init_app(app)

    def init_app(self, app):
        app.config.setdefault("EXTEND_HEADERS_KEY", "Accept")

    @classmethod
    def register(cls, extensions: dict, default: str = ""):
        def decorator(func):
            @functools.wraps(func)
            def wrapper(*args, **kwargs):
                custom_header_key = current_app.config["EXTEND_HEADERS_KEY"].title()
                request_header_keys = [key for key in request.headers.keys()]
                if (
                    default
                    and custom_header_key in request_header_keys
                    and request.headers[custom_header_key] == default
                ):
                    return func(*args, **kwargs)
                elif custom_header_key in request_header_keys:
                    for extension in extensions.keys():
                        if extension == request.headers[custom_header_key]:
                            return extensions[extension](*args, **kwargs)
                elif not default:
                    return func(*args, **kwargs)
                description = f"{NotAcceptable.description} Supported entities are: {custom_header_key}"
                raise NotAcceptable(description)

            return wrapper

        return decorator



import json


import pytest


from flask_app_for_testing import app


def test_no_default():
    with app.test_client() as client:
        req = client.get("/no_default_endpoint")
        assert req.status_code == 200
        assert "v0" == json.loads(req.data.decode())["version"]


@pytest.mark.parametrize(
    "headers,status_code,version",
    [
        ("application/v1", 200, "v1"),
        ("application/v2", 200, "v2"),
    ],
)
def test_no_default_versioning(headers, status_code, version):
    with app.test_client() as client:
        req = client.get("/no_default_endpoint", headers={"accept-version": headers})
        assert req.status_code == status_code
        assert version == json.loads(req.data.decode())["version"]


@pytest.mark.parametrize(
    "headers,status_code",
    [
        ("application/v3", 406),
        ("", 406),
    ],
)
def test_no_default_not_acceptable(headers, status_code):
    with app.test_client() as client:
        req = client.get("/no_default_endpoint", headers={"accept-version": headers})
        assert req.status_code == status_code


@pytest.mark.parametrize(
    "headers,status_code,version",
    [
        ("application/v1", 200, "v1"),
        ("application/v2", 200, "v2"),
        ("application/v3", 200, "v3"),
    ],
)
def test_default_versioning(headers, status_code, version):
    with app.test_client() as client:
        req = client.get("/default_endpoint", headers={"accept-version": headers})
        assert req.status_code == status_code
        assert version == json.loads(req.data.decode())["version"]


@pytest.mark.parametrize(
    "headers,status_code",
    [
        ("application/v4", 406),
        ("", 406),
    ],
)
def test_default_not_acceptable(headers, status_code):
    with app.test_client() as client:
        req = client.get("/default_endpoint", headers={"accept-version": headers})
        assert req.status_code == status_code



from flask import Flask
from flask_extend_headers import ExtendHeaders


app = Flask(__name__)
app.debug = True

app.config["EXTEND_HEADERS_KEY"] = "accept-version"

extend_headers = ExtendHeaders(app)


def no_default_endpoint_v1():
    return {"version": "v1"}, 200


def no_default_endpoint_v2():
    return {"version": "v2"}, 200


@app.route("/no_default_endpoint")
@extend_headers.register(
    extensions={
        "application/v1": no_default_endpoint_v1,
        "application/v2": no_default_endpoint_v2,
    }
)
def no_default_endpoint():
    return {"version": "v0"}, 200


def default_endpoint_v2():
    return {"version": "v2"}, 200


def default_endpoint_v3():
    return {"version": "v3"}, 200


@app.route("/default_endpoint")
@extend_headers.register(
    extensions={
        "application/v2": default_endpoint_v2,
        "application/v3": default_endpoint_v3,
    },
    default="application/v1",
)
def default_endpoint_v1():
    return {"version": "v1"}, 200



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['flask_extend_headers']

package_data = \
{'': ['*']}

install_requires = \
['flask>=0.12.2']

setup_kwargs = {
    'name': 'flask-extend-headers',
    'version': '1.1.1',
    'description': 'Flask extend headers module for API versioning.',
    'long_description': '# flask-extend-headers\n\nFlask extend headers module for API versioning.\n\n[![PyPI version](https://badge.fury.io/py/flask-extend-headers.svg)](https://badge.fury.io/py/flask-extend-headers)\n\n## Description\n\nCustom header routing for versioning your Flask API.\n\n## Features\n\n* Use it as a decorator\n* Return with different views based on the custom header you set\n\n## Documentation\n\n### Installation\n\n```bash\npip install flask-extend-headers\n```\n\n### Quickstart\n\nBelow is an example of two API endpoints with different URL with version:\n\n```python\nfrom flask import Flask\n\napp = Flask(__name__)\n\n@app.route(\'/api/v2/users\')\ndef usersv2():\n    return "usersv2", 200\n\n@app.route(\'/api/v1/users\')\ndef users():\n    return "users", 200\n\nif __name__ == \'__main__\':\n    app.run()\n```\n\nWe could change this implementation using `flask_extend_headers` and specifying the version in the `headers`\n\n```python\nfrom flask import Flask\nfrom flask_extend_headers import ExtendHeaders\n\napp = Flask(__name__)\n\napp.config["EXTEND_HEADERS_KEY"] = "accept-version"\n\nextend_headers = ExtendHeaders(app)\n\ndef usersv2():\n    return "usersv2", 200\n\n@app.route(\'/api/users\')\n@extend_headers.register(\n    extensions={\n        "application/v2": usersv2,\n    },\n    default="application/v1"\n)\ndef users():\n    return "users", 200\n\nif __name__ == \'__main__\':\n    app.run()\n```\n\nIf we call this API it\'ll return a `406`:\n\n```bash\n> curl http://localhost:5000/api/users -I\nHTTP/1.0 406 NOT ACCEPTABLE\nContent-Type: text/html; charset=utf-8\nContent-Length: 350\nServer: Werkzeug/2.0.2 Python/3.9.6\nDate: Mon, 18 Oct 2021 19:33:46 GMT\n```\n\nIf we add the headers it\'ll return `users`:\n\n```bash\n> curl http://localhost:5000/api/users -I -H "Accept-Version: application/v1"\nHTTP/1.0 200 OK\nContent-Type: text/html; charset=utf-8\nContent-Length: 5\nServer: Werkzeug/2.0.2 Python/3.9.6\nDate: Mon, 18 Oct 2021 19:34:55 GMT\n```\n\nIf we modify the headers it\'ll return `usersv2`\n```bash\n> curl http://localhost:5000/api/users -I -H "Accept-Version: application/v2"\nHTTP/1.0 200 OK\nContent-Type: text/html; charset=utf-8\nContent-Length: 5\nServer: Werkzeug/2.0.2 Python/3.9.6\nDate: Mon, 18 Oct 2021 19:35:48 GMT\n```\n\n\n### Fallback on view\n\nIf you have a new version of a view but you want to fallback in a view without a specified version:\n\n```python\ndef usersv2():\n    return "usersv2", 200\n\n@app.route(\'/api/users\')\n@extend_headers.register(\n    extensions={\n        "application/v2": usersv2,\n    }\n)\ndef users():\n    return "users", 200\n```\n\nIf we call the endpoint without headers it\'ll return the fallback view `users`:\n\n```bash\n> curl http://localhost:5000/api/users -I\nHTTP/1.0 200 OK\nContent-Type: text/html; charset=utf-8\nContent-Length: 5\nServer: Werkzeug/2.0.2 Python/3.9.6\nDate: Mon, 18 Oct 2021 19:42:05 GMT\n```\n\nIf we call the endpoint specifying headers it\'ll return `usersv2`:\n\n```bash\n> curl http://localhost:5000/api/users -I -H "Accept-Version: application/v2"\nHTTP/1.0 200 OK\nContent-Type: text/html; charset=utf-8\nContent-Length: 7\nServer: Werkzeug/2.0.2 Python/3.9.6\nDate: Mon, 18 Oct 2021 19:42:36 GMT\n```\n\n## Testing\n\nInstall `poetry` and execute `pytest-cov`\n\n```bash\npip install poetry\npoetry install\npoetry run pytest --cov=flask_extend_headers tests\n```\n\n## License\n\nMIT\n',
    'author': 'Luis Emilio Moreno',
    'author_email': 'emilio@touchof.tech',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/lemiliomoreno/flask-extend-headers',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
